@@ -1,20 +1,17 @@
 '''
 Luke-zhang-04
-Canvas Plus v1.1.3 (https://github.com/Luke-zhang-04/CanvasPlus)
-Copyright (C) 2020 Luke-zhang-04
+CanvasPlus v1.2.0 (https://github.com/Luke-zhang-04/CanvasPlus)
+Copyright (C) 2020 Luke Zhang
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program. If not, see https://github.com/Luke-zhang-04/CanvasPlus/blob/master/LICENSE.
+The above copyright notice and this permission notice shall be included in all
+copies or substantial portions of the Software.
 '''
 
 #tkinter
@@ -35,7 +32,12 @@ from typing import Tuple, Union, List, Callable, Dict
 #warnings
 import warnings
 
-print("Hello from CanvasPlus")
+#regex
+import re
+
+_canvasPlusVersion = "v1.2.0"
+
+print("This is CanvasPlus %s" % _canvasPlusVersion)
 
 class Error(Exception):
    '''Base class for other exceptions'''
@@ -53,6 +55,10 @@ class InvalidObjectType(Error):
     '''raised when object type not supported'''
     pass
 
+class InvalidEquation(Error):
+    '''raised when euqtion of a line is invalid'''
+    pass
+
 
 class WidgetWindows:
     '''Class for createing widgets as windows within the canvas'''
@@ -170,8 +176,79 @@ class WidgetWindows:
         return self._create_widget(x, y, Spinbox, **kwargs)
 
 
+class AnalyticGeometry:
+    '''perform basic analytic geometry'''
+
+    @staticmethod
+    def make_eqn(slope: Union[float, int, None], *pt) -> Dict:
+        '''gets the parts of an equation'''
+        properties = {}
+        if slope == None: #got errors becase 0 evaulates to False
+            properties = {
+                "m": None,
+                "x": pt[0]
+            }
+        
+        elif slope == 0:
+            properties = {
+                "m": 0,
+                "y": pt[1]
+            }
+        
+        else:
+            properties = {
+                "m": slope,
+                "b": pt[1]-slope*pt[0]
+            }
+
+        return properties
+
+    @staticmethod
+    def perpendicular_slope(eqn: Dict) -> Union[float, int, None]:
+        '''gets the perpendicular slope of an equation'''
+        if "m" not in eqn or not eqn["m"]:
+            if "y" in eqn:
+                perpendicular = None
+            elif "x" in eqn:
+                perpendicular = 0
+        else:
+            if float(eqn["m"]) == 0: perpendicular = None
+            else: perpendicular = -(1/float(eqn["m"]))
+        return perpendicular
+
+    @staticmethod
+    def get_poi(eqn1: Dict, eqn2: Dict) -> Tuple[Union[float, int]]:
+        '''gets the point of intersection between two lines'''
+        poi = ()
+
+        flat = False
+        if "x" in eqn1:
+            flat = eqn1["m"] in (None, 0) and eqn2["m"] in (None, 0)
+        elif "y" in eqn1:
+            flat = eqn2["m"] in (None, 0) and eqn1["m"] in (None, 0)
+
+        if flat:
+            if "x" in eqn1:
+                poi = float(eqn1["x"]), float(eqn2["y"])
+            else:
+                poi = float(eqn2["x"]), float(eqn1["y"])
+        else:
+            if "b" not in eqn1: eqn1["b"] = 0
+            if "b" not in eqn2: eqn2["b"] = 0
+            x = (
+                (float(eqn1["b"])-float(eqn2["b"])) /
+                (float(eqn2["m"])-float(eqn1["m"]))
+            )
+            y = float(eqn1["m"])*x + float(eqn1["b"])
+            poi = (x, y)
+
+        return poi
+
+
 class Transformations:
-    def rotate(self, tagOrId: Union[int, str], x: Real, y: Real, amount: Real, unit: str = "rad", warn: bool = True) -> None:
+    '''define transformation methods'''
+
+    def rotate(self, tagOrId: Union[int, str], x: Real, y: Real, amount: Real, unit: str = "rad", warn: bool = True) -> Tuple[Union[float, int]]:
         '''rotate obj on axis x, y by amount in degrees or radians clockwise'''
         if unit in ("d", "deg", "degree", "degrees"):
             amount *= math.pi/180 #convert to radians
@@ -202,9 +279,74 @@ class Transformations:
                     UnsupportedObjectType
                 )
             self.coords(tagOrId, *newCords)
+        return newCords
+
+    def flip(self, tagOrId: Union[int, str], **eqn: Dict) -> Tuple[Union[float, int]]:
+        '''flips tagOrId on line eqn. eqn should be either {y: val}, {x: val}, or {m: val, b: val} m being slope and b being y-intercept'''
+        if len(eqn) == 0: raise InvalidEquation("Empty equation")
+
+        for key, _ in eqn.items():
+            if key != "x": eqn[key] = float(eqn[key]) * -1
+
+        if "x" in eqn and "m" not in eqn:
+            eqn["m"] = None
+        elif "y" in eqn and "m" not in eqn:
+            eqn["m"] = 0
+
+        all_cords = self.coords(tagOrId)
+        cords = [
+            (all_cords[i], all_cords[i+1]) for i in range(0, len(all_cords), 2)
+        ]
+
+        #perpendicular slope
+        perSlope = AnalyticGeometry.perpendicular_slope(eqn)
+
+        #perpendicular eqnations
+        perEqns = [AnalyticGeometry.make_eqn(perSlope, i[0], -i[1]) for i in cords]
+
+        #Points of intersect
+        POIs = [AnalyticGeometry.get_poi(eqn, i) for i in perEqns]
+
+        newPts = [] #new points
+
+        for i in range(len(POIs)): #get new points
+            newPts.append((
+                POIs[i][0]-(cords[i][0]-POIs[i][0]),
+                (-(POIs[i][1])-cords[i][1])-POIs[i][1]
+            ))
+        
+        newCords = []
+        for i in newPts:
+            newCords.append(i[0])
+            newCords.append(i[1])
+        
+        self.coords(tagOrId, *newCords)
+        return newPts
+        
+    reflect = flip
+
+    def resize(self, tagOrId: Union[int, str], scale: Real, x: Real, y: Real) -> Tuple[Union[float, int]]:
+        '''Resizes tagOrId by scale with point x, y'''
+        vals = self.coords(tagOrId)
+        coords = [(vals[i], vals[i+1]) for i in range(0, len(vals), 2)]
+        newCoords = []
+
+        if scale < 1:
+            for x1, y1 in coords:
+                newCoords.append(x1 + (x - x1)*scale)
+                newCoords.append(y1 - (y1 - y)*scale)
+        elif scale > 1:
+            for x1, y1 in coords:
+                newCoords.append(x - (x - x1)*scale)
+                newCoords.append(y - (y - y1)*scale)
+        
+        print(newCoords)
+        return newCoords
+    
+    size = scale = resize 
 
 
-class CanvasPlus(Canvas, WidgetWindowsm, Transformations):
+class CanvasPlus(Canvas, WidgetWindows, Transformations):
     '''Improved Canvas widget with more functionality to display graphical elements like lines or text.'''
 
     def clone(self, tagOrId: Union[int, str], *args: List[int]) -> int:
@@ -304,9 +446,8 @@ class CanvasPlus(Canvas, WidgetWindowsm, Transformations):
     poly = to_polygon
 
     def tags_bind(
-        self, tagsOrIds: Union[Union[int, str], Tuple[Union[int, str]], List[Union[int, str]]],
-        sequences: Union[str, Tuple[str], List[str]] = None,
-        funcs = Union[Callable, Tuple[Callable], List[Callable]], add = None) -> Union[str, List[str]]:
+        self, tagsOrIds: Union[int, str, Tuple], sequences: Union[str, Tuple] = None,
+        funcs = Union[Callable, Tuple], add: bool = None) -> Union[str, List[str]]:
         '''Binds either multiple tags to one function, or multiple tags to multiple functions with matching indicies in one function
         
         i.e (tag1, tag2, tag3), func1 will bind tag1, tag2, tag3 into fun1, while (tag1, tag2, tag3), (func1, func2, func3) will bind tag1 to func1, tag2 to func2, tag3 to func3.
@@ -340,7 +481,6 @@ def _test():
     root = Tk()
     canvas = CanvasPlus(root, width=800, height=800, background = "white")
     canvas.pack()
-
     #create circle function
     canvas.create_circle(300, 600, 100, fill = "black", outline = "green", width = 3)
     
@@ -362,7 +502,7 @@ def _test():
     #create an entry and set it's default value
     content = StringVar()
     canvas.create_entry(0, 0, anchor = "nw", textvariable = content, fg = "blue", bg = "gold")
-    content.set("This is CanvasPlus v1.1.3")
+    content.set("This is CanvasPlus %s" % _canvasPlusVersion)
 
     #create button to print the value in the previously cretaed entry
     canvas.create_button(
@@ -381,10 +521,13 @@ def _test():
         5, 75, font = ("Times", "24"), fg = "black", bg = "green", text = "By Luke-zhang-04", anchor = "nw"
     )
 
+    #flip example
+    aPrime = canvas.create_polygon(500, 10, 500, 20, 550, 25, 600, 20, 600, 10, fill = "yellow", outline = "black")
+    a = canvas.clone(aPrime)
+    canvas.flip(a, m = .5, b = -200)
+
     canvas.update()
     canvas.mainloop()
 
 if __name__ == "__main__":
-    _test()
-
-print("Imported successfully")
\ No newline at end of file
+    _test()
\ No newline at end of file
